require 'json'
require 'uu/os/gvc/component'
require 'uu/os/gvc/message'
require 'uu/os/gvc/icon'
require 'uu/os/gvc/image'
require 'uu/os/gvc/table'
require 'uu/os/gvc/button'
require 'uu/os/gvc/drop_down_button'
require 'uu/os/gvc/link'
require 'uu/os/gvc/text'
require 'uu/os/gvc/touch_icon'
require 'uu/os/gvc/tree'
require 'uu/os/gvc/widget'
require 'uu/os/form'
require 'uu/os/gvc/uxmlfragment'

module UU
  module OS
    module GVC

      # Abstract container for managing components.
      class Container

        # Creates new instance of container.
        #
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(data = nil)
          @components = {}
          @messages = []
          @focus_on = nil
          if data.nil?
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if !data.kind_of?Hash
            raise ArgumentError.new("Container data must be JSON String or Hash, but was #{data.class}.")
          end
          if data[:components]
            data[:components].each do |cmp|
              component = create_component(cmp)
              if (@components[component.id])
                raise ArgumentError.new('Invalid container data. Container contains multiple components with same ID.')
              end
              @components[component.id] = component
            end
          end
          if data[:messages]
            data[:messages].each do |message|
              add_message(message)
            end
          end
        end

        # Finds single component matching given unique ID.
        #
        # @param [String] id Unique component ID
        #
        # @return [UU::OS::GVC::Component] Existing component or nil if no component found
        def get_component_by_id(id)
          @components[id]
        end

        # Finds all components matching given code. If nil is entered as the code, than all components
        # without a defined code are returned.
        #
        # @param [String] code Component code
        #
        # @return [Array<UU::OS::GVC::Component>] List of matching components (empty list if no components found)
        def get_components_by_code(code)
          result = []
          @components.values.each do |component|
            result << component if component.code == code
          end
          result
        end

        # Returns list of all components in container.
        #
        # @return [Array<UU::OS::GVC::Component>] List of all components
        def components
          @components.values
        end

        # Adds global container message.
        #
        # @param [UU::OS::GVC::Message] message Instance of message
        def add_message(message)
          msg = UU::OS::GVC::Message.new(message)
          @messages << msg
        end

        # Returns list of all messages in container (both global and component related).
        #
        # @return [Array<UU::OS::GVC::Message>] List of all messages
        def messages
          # Disallow direct manipulating with messages
          @messages.clone
        end

        # Sets focus to component with given ID. Actual behavior of focus depends
        # on type of focused component and also on type of container in which is
        # component placed.
        # Only one component can be focused, setting focus on one component
        # disables focus of previously focused component within container.
        #
        # @param [String] id Unique component ID
        #
        # @return [FalseClass, TrueClass] True if focus was set, else false
        def focus(component_id)
          return false if !get_component_by_id(component_id)
          @focus_on = component_id
          return true
        end

        # Returns component marked as focused. Nil in case no focus is set to any component.
        #
        # @return[UU::OS::GVC::Component] Focused component or nil
        def focused_component
          @components[@focus_on]
        end

        private

        # TODO Component registry should be used

        # @private
        @@COMPONENT_MAPPING = {
          'uu.os.form.infobar' => UU::OS::Form::InfoBar,
          'uu.os.form.label' => UU::OS::Form::Label,
          'uu.os.form.input' => UU::OS::Form::Input,
          'uu.os.form.date' => UU::OS::Form::Date,
          'uu.os.form.textarea' => UU::OS::Form::Textarea,
          'uu.os.form.text' => UU::OS::Form::Text,
          'uu.os.form.choice' => UU::OS::Form::Choice,
          'uu.os.form.radio' => UU::OS::Form::Radio,
          'uu.os.form.reference' => UU::OS::Form::Reference,
          'uu.os.form.formbutton' => UU::OS::Form::FormButton,
          'uu.os.form.checkbox' => UU::OS::Form::Checkbox,
          'uu.os.form.file' => UU::OS::Form::File,
          'uu.os.form.inputimage' => UU::OS::Form::InputImage,
          'uu.os.form.iconpicker' => UU::OS::Form::IconPicker,
          'uu.os.form.colorpicker' => UU::OS::Form::ColorPicker,

          'uu.os.gvc.button' => UU::OS::GVC::Button,
          'uu.os.gvc.dropdownbutton' => UU::OS::GVC::DropDownButton,
          'uu.os.gvc.icon' => UU::OS::GVC::Icon,
          'uu.os.gvc.image' => UU::OS::GVC::Image,
          'uu.os.gvc.link' => UU::OS::GVC::Link,
          'uu.os.gvc.table' => UU::OS::GVC::Table,
          'uu.os.gvc.text' => UU::OS::GVC::Text,
          'uu.os.gvc.tree' => UU::OS::GVC::Tree,
          'uu.os.gvc.touchicon' => UU::OS::GVC::TouchIcon,
          'uu.os.gvc.uxmlfragment' => UU::OS::GVC::UxmlFragment,
          'uu.os.gvc.widget' => UU::OS::GVC::Widget
        }

        # @private
        def create_component(data)
          if data
            comp_cls = @@COMPONENT_MAPPING[data[:componentType].to_s.downcase]
            if comp_cls
              # Create component according to mapping
              return comp_cls.new(self, data)
            elsif !data[:value].nil? || !data[:data].nil?
              # If no mapping exists but component contains value or data, it is probably Input component
              return UU::OS::Form::Input.new(self, data)
            end
          end
          # Create generic component by default
          UU::OS::GVC::Component.new(self, data)
        end

      end

    end
  end
end
